package at.jku.fim.datalinksimulation.networkstack;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.networkstack.incoming.IncomingNetworkStack;
import at.jku.fim.datalinksimulation.networkstack.incoming.IncomingNetworkStackConfiguration;
import at.jku.fim.datalinksimulation.networkstack.incoming.IncomingNetworkStackObserver;
import at.jku.fim.datalinksimulation.networkstack.outgoing.OutgoingNetworkStack;
import at.jku.fim.datalinksimulation.networkstack.outgoing.OutgoingNetworkStackConfiguration;
import at.jku.fim.datalinksimulation.networkstack.outgoing.OutgoingNetworkStackObserver;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: NetworkStack.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */
public class NetworkStack {

    /**
     * Network stack parameter name for frame sender
     */
    public static String NETWORK_PARAM_SENDER_ADDRESS = "senderAddress";

    /**
     * Network stack parameter name for frame receiver
     */
    public static String NETWORK_PARAM_RECEIVER_ADDRESS = "receiverAddress";

    /**
     * Network stack parameter name for incoming port number
     */
    public static String NETWORK_PARAM_INCOMING_PORT_NUM = "incomingPortNumber";
    
    /**
     * Network stack parameter name for outgoing port number;
     * Value is null or List containing "Integer" objects for ports
     */
    public static String NETWORK_PARAM_OUTGOING_PORT_NUMS = "outgoingPortNumbers";
    
    /**
     * Associated Incoming Network Stack  
     */
    protected IncomingNetworkStack incomingNetworkStack;

    /**
     * Associated Outgoing Network Stack
     */
    protected OutgoingNetworkStack outgoingNetworkStack;

    /**
     * Station identifier
     */
    protected Object stationID = null;

    /**
     * Constructor
     * @param id Station identifier
     */
    public NetworkStack(
    		IncomingNetworkStackConfiguration incomingConfig,
    		OutgoingNetworkStackConfiguration outgoingConfig,
    		Object id) {
    	
        incomingNetworkStack = new IncomingNetworkStack(incomingConfig, id);
        outgoingNetworkStack = new OutgoingNetworkStack(outgoingConfig, id);
        
        setStationID(id);
    }

    /**
     * Add application
     * @param application
     */
    public void subscribeApplication(IncomingNetworkStackObserver application) {
    	incomingNetworkStack.subscribe(application);
    }

    /**
     * Remove application
     * @param application
     */
    public void unsubscribeApplication(IncomingNetworkStackObserver application) {
    	incomingNetworkStack.unsubscribe(application);
    }
    
    public void connectPhysical(OutgoingNetworkStackObserver plug) {
    	outgoingNetworkStack.subscribe(plug);
    }

    public void disconnectPhysical(OutgoingNetworkStackObserver plug) {
    	outgoingNetworkStack.unsubscribe(plug);
    }
    

	/**
	 * Send data to world 
	 * @param receiver
	 * @param data
	 * @param networkStackParameters
	 */
    public void sendToWorld(Object receiver, RawData data, NetworkStackParameters networkStackParameters) {
    	if (networkStackParameters == null)
    		networkStackParameters = new NetworkStackParameters();
    	
    	if (networkStackParameters.getParameter(
        		NetworkStackParameters.GLOBAL_PARAMETER,
                NETWORK_PARAM_SENDER_ADDRESS) == null) {
            networkStackParameters.setParameter(
                    NetworkStackParameters.GLOBAL_PARAMETER,
                    NETWORK_PARAM_SENDER_ADDRESS, stationID);
    	}
    	
    	if (networkStackParameters.getParameter(
        		NetworkStackParameters.GLOBAL_PARAMETER,
        		NETWORK_PARAM_RECEIVER_ADDRESS) == null) {
            networkStackParameters.setParameter(
                    NetworkStackParameters.GLOBAL_PARAMETER,
                    NETWORK_PARAM_RECEIVER_ADDRESS, receiver);
    	}
    	
    	outgoingNetworkStack.send(networkStackParameters, data);
    }
    
    public void receiveFromWorld(RawData data, int port) {
    	NetworkStackParameters params = new NetworkStackParameters();
    	params.setParameter(
    			1,
    			NETWORK_PARAM_INCOMING_PORT_NUM, 
    			new Integer(port));
    	
    	incomingNetworkStack.receiveNewData(data, params);
    }
    
    /**
     * @return Returns the stationID.
     */
    public Object getStationID() {
        return stationID;
    }

    /**
     * @param stationID The stationID to set.
     */
    public void setStationID(Object stationID) {
        this.stationID = stationID;
        this.incomingNetworkStack.setStationId(stationID);
        this.outgoingNetworkStack.setStationID(stationID);
    }

    /**
	 * @param parent Parent logger
	 */
    public void setParentLogger(Logger parent) {
    	incomingNetworkStack.setParentLogger(parent);
    	outgoingNetworkStack.setParentLogger(parent);
    }

}